from rest_framework.permissions import DjangoModelPermissions


class AdminPermissions(DjangoModelPermissions):
    def __init__(self):
        self.perms_map['GET'] = ['%(app_label)s.view_%(model_name)s']



from django.db import models
from fwbasemodel.querysets import BaseQuerySet


class BaseManager(models.Manager):
    def get_queryset(self):

        return BaseQuerySet(self.model, using=self._db).filter(deleted=False)


class AdminManager(models.Manager):
    def get_queryset(self):
        return BaseQuerySet(self.model, using=self._db)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import logging
from copy import deepcopy
from django.db import models
from django.contrib.auth import get_user_model
from django.forms import model_to_dict
from django.utils import timezone
from fwbasemodel.managers import BaseManager, AdminManager

User = get_user_model()

PERMISSION_EXPORT_CODE = "app_{}_can_export"
PERMISSION_EXPORT_NAME = "Puede exportar"


class _Model(models.Model):
    created_at = models.DateTimeField(auto_now_add=True, editable=False, verbose_name='Creado el')

    created_by = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name='Creado por',
                                   null=True, blank=True, related_name='%(app_label)s_%(class)s_created_by')

    deleted = models.BooleanField('Eliminado', default=False)

    deleted_at = models.DateTimeField('Eliminado el', null=True, blank=True)

    def save(self, *args, **kwargs):
        if kwargs.get('request', None) and self.created_by is None:
            request = kwargs.pop('request')
            self.created_by = request.user
        super(_Model, self).save()

    def delete(self, *args, **kwargs):
        self.deleted = True
        self.deleted_at = timezone.now()
        self.save(*args, **kwargs)

    class Meta:
        abstract = True


class BaseModel(_Model):
    """
    ;log_class: LogModel define una clase para log la cual debe extender de LogModel
    """

    updated_at = models.DateTimeField(auto_now=True, verbose_name='Actualizado el')

    active = models.BooleanField('Activo', default=True, help_text='Indica si el modelo está operativo')

    objects = BaseManager()

    admin_objects = AdminManager()

    log_class = None

    EXCLUDES = ['created_by', 'created_at', 'updated_at']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        
        if self.log_class:
            self.__initial = deepcopy(self)

    def save(self, *args, **kwargs):
        super(BaseModel, self).save(*args, **kwargs)

        adding = self._state.adding

        # Preguntar si el modelo tiene o no clase para dejar logs de cambios.
        if not self.log_class or not adding:
            return

        current = model_to_dict(self)
        initial = model_to_dict(self.__initial)

        for field in self._meta.fields:
            if field.name in self.EXCLUDES or field.name not in current:
                continue

            # Preguntar si el valor del campo cambio
            if current[field.name] != initial[field.name]:
                if isinstance(field, models.TextField):
                    before_text = None
                    after_text = None

                    if getattr(self.__initial, field.name):
                        before_text = str(getattr(self.__initial, field.name))
                    if getattr(self, field.name):
                        after_text = str(getattr(self, field.name))

                    # Agregar el registro con el cambio a la clase de logs.
                    log_class = self.log_class(field=field.name,
                                               before_text=before_text,
                                               after_text=after_text,
                                               record=self)
                else:
                    before_char = None
                    after_char = None

                    if getattr(self.__initial, field.name):
                        try:
                            before_char = str(getattr(self.__initial, 'get_' + field.name + '_display')())
                        except Exception as e:
                            logging.exception(e)
                            before_char = str(getattr(self.__initial, field.name))

                    if getattr(self, field.name):
                        try:
                            after_char = str(getattr(self, 'get_' + field.name + '_display')())
                        except Exception as e:
                            logging.exception(e)
                            after_char = str(getattr(self, field.name))

                    # Agregar el registro con el cambio a la clase de logs.
                    log_class = self.log_class(field=field.name,
                                               before_char=before_char,
                                               after_char=after_char,
                                               record=self)
                log_class.save()

        self.__initial = deepcopy(self)

    @staticmethod
    def export_permissions(name: str = '') -> list:
        """
        :param name: str
        :return: list
        """
        return [(PERMISSION_EXPORT_CODE.format(str(name).lower()), PERMISSION_EXPORT_NAME)]

    class Meta:
        abstract = True


class LogModel(_Model):
    """
    Clase base para todos los modelos del aplicativo
    """
    field = models.CharField('Campo', max_length=150, null=True)
    before_char = models.CharField('Texto anterior', max_length=500, null=True, blank=True)
    after_char = models.CharField('Texto posterior', max_length=500, null=True, blank=True)
    before_text = models.TextField('Texto largo anterior', null=True, blank=True)
    after_text = models.TextField('Texto largo posterior', null=True, blank=True)
    before_id = models.IntegerField('Id anterior', null=True, blank=True)
    after_id = models.IntegerField('Id posterior', null=True, blank=True)

    objects = BaseManager()

    class Meta:
        abstract = True



from django.conf import Settings
from rest_framework import pagination
from rest_framework.response import Response


class RestPagination(pagination.PageNumberPagination):

    page_size_query_param = getattr(Settings, 'PAGE_PARAM_NAME', 'page_size')
    page_size = getattr(Settings, 'PAGINATION_SIZE', 10)

    def get_paginated_response(self, data):

        return Response({
            'next': self.get_next_link(),
            'previous': self.get_previous_link(),
            'count': self.page.paginator.count,
            'num_pages': self.page.paginator.num_pages,
            'current_page': int(self.request.query_params.get('page', 1)),
            'results': data
        })



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='fwbasemodel',
    version='1.0.5',
    author='Germapat',
    author_email='gdpm1986@gmail.com',
    description='Base models, export',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/germapat/fw',
    packages=setuptools.find_packages(exclude=['tests*']),
    include_package_data=True,
    install_requires=[],
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ]

)



# ACTION_CHECKBOX_NAME is unused, but should stay since its import from here
# has been referenced in documentation.

from django.contrib.admin.sites import site
from django.utils.module_loading import autodiscover_modules
from fwbasemodeladmin.options import BaseModelAdmin, BaseInlineMixin

__all__ = ['BaseModelAdmin', 'BaseInlineMixin']


def autodiscover():
    autodiscover_modules('admin', register_to=site)


default_app_config = 'django.contrib.admin.apps.AdminConfig'
